/*******************************************************************
    Copyright (C) 2009 FreakLabs
    All rights reserved.

    Redistribution and use in source and binary forms, with or without
    modification, are permitted provided that the following conditions
    are met:

    1. Redistributions of source code must retain the above copyright
       notice, this list of conditions and the following disclaimer.
    2. Redistributions in binary form must reproduce the above copyright
       notice, this list of conditions and the following disclaimer in the
       documentation and/or other materials provided with the distribution.
    3. Neither the name of the the copyright holder nor the names of its contributors
       may be used to endorse or promote products derived from this software
       without specific prior written permission.

    THIS SOFTWARE IS PROVIDED BY THE THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS'' AND
    ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
    IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
    ARE DISCLAIMED.  IN NO EVENT SHALL THE INSTITUTE OR CONTRIBUTORS BE LIABLE
    FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
    DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
    OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
    HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
    LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
    OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
    SUCH DAMAGE.

    Originally written by Christopher Wang aka Akiba.
    Please post support questions to the FreakLabs forum.
*******************************************************************/

/**************************************************************************/
/*!
    @file     cli.c
    @author   Christopher Wang (Freaklabs)
              Modified by: K. Townsend (microBuilder.eu)

    @ingroup  CLI

    Original code taken from the FreakUSB Open Source USB Device Stack
    http://freaklabs.org/index.php/FreakUSB-Open-Source-USB-Device-Stack.html

    If it works well, you can thank Akiba at Freaklabs.  If it fails
    miserably, you can blame me (since parts of it it were rather
    ungraciously modified). :-)

*/
/**************************************************************************/

#include <stdio.h>
#include <string.h>

#include "cli.h"
#include "cli_tbl.h"
#include "commands.h"

#include "cdc.h"

#define KEY_CODE_ESC        (27)    /* Escape key code */
#define KEY_CODE_ENTER      (13)    /* Enter key code  */

static uint8_t msg[50];
static uint8_t *msg_ptr;

/**************************************************************************/
/*!
    @brief  Polls the relevant incoming message queue to see if anything
            is waiting to be processed.
*/
/**************************************************************************/
void CLI_Task(void)
{
  if (CDC_USB_Configured())
  {
		while (1)
		{
			int c = getchar();
			if (c == EOF)
				break;
			
			cliRx(c);
    }
  }
}

#if 0
/**************************************************************************/
/*!
    @brief  Reads the CLI input until the 'Enter' key is detected, or
            until we reach the end of the input buffer (use with care!)
*/
/**************************************************************************/
void cliReadLine(uint8_t *str, uint16_t *strLen)
{
  uint8_t ch;
  uint16_t idx = 0;

  /* ToDo: Update this to handle UART, etc., with proper #ifdef blocks! */

  while (1)
  {
    if (CDC_USB_Configured())
    {
      while (getchar() != EOF);

      if ((ch >= 32) && (ch <= 126) && (idx < *strLen))
      {
        str[idx++] = ch;
        cliRx(ch);
      }
      if (((ch == 8) || (ch == 127)) && idx)
      {
        str[idx--] = 0;
        cliRx(ch);
      }
      if (ch == KEY_CODE_ENTER)
      {
        *strLen = idx;
        break;
      }
    }
  }
}
#endif

/**************************************************************************/
/*!
    @brief  Handles a single incoming character.  If a new line is
            detected, the entire command will be passed to the command
            parser.  If a text character is detected, it will be added to
            the message buffer until a new line is detected (up to the
            maximum queue size, CFG_INTERFACE_MAXMSGSIZE).

    @param[in]  c
                The character to parse.
*/
/**************************************************************************/
void cliRx(uint8_t c)
{
  // read out the data in the buffer and echo it back to the host.
  switch (c)
  {
    case '\r':
//      break;
    case '\n':
        // terminate the msg and reset the msg ptr. then send
        // it to the handler for processing.
        *msg_ptr = '\0';
        printf("\r\n");
        cliParse((char *)msg);
        msg_ptr = msg;
        break;

    case '\b':
        printf("%c",c);
        if (msg_ptr == msg)
        {
            // Send bell alert and space (to maintain position)
            printf("\a ");
        }
        else if (msg_ptr > msg)
        {
            msg_ptr--;
        }
        break;

    default:
        printf("%c",c);
        *msg_ptr++ = c;
        break;
  }
}

/**************************************************************************/
/*!
    @brief  Displays the command prompt.  The text that appears is defined
            in projectconfig.h.
*/
/**************************************************************************/
static void cliMenu()
{
  printf("\r\n");
  printf(">");
}

/**************************************************************************/
/*!
    @brief  Parse the command line. This function tokenizes the command
            input, then searches for the command table entry associated
            with the commmand. Once found, it will jump to the
            corresponding function.

    @param[in]  cmd
                The entire command string to be parsed
*/
/**************************************************************************/
void cliParse(char *cmd)
{
  size_t argc, i = 0;
  char *argv[30];

  argv[i] = strtok(cmd, " ");
  do
  {
      argv[++i] = strtok(NULL, " ");
  } while ((i < 30) && (argv[i] != NULL));

  argc = i;
  for (i=0; i < CMD_COUNT; i++)
  {
      if (!strcmp(argv[0], cli_tbl[i].command))
      {
        if ((argc == 2) && !strcmp (argv [1], "?"))
        {
          // Display parameter help menu on 'command ?'
          printf ("%s\r\n\r\n", cli_tbl[i].description);
          printf ("%s\r\n", cli_tbl[i].parameters);
        }
        else if ((argc - 1) < cli_tbl[i].minArgs)
        {
          // Too few arguments supplied
          printf ("Too few arguments\r\n");
        }
        else if ((argc - 1) > cli_tbl[i].maxArgs)
        {
          // Too many arguments supplied
          printf ("Too many arguments\r\n");
        }
        else
        {
          // Dispatch command to the appropriate function
          cli_tbl[i].func(argc - 1, &argv [1]);
        }

        // Refresh the command prompt
        cliMenu();
        return;
      }
  }
  // Command not recognized
  printf ("Command Not Recognised\r\n");
  printf ("Type '?' for a list of all available commands\r\n");

  cliMenu();
}

/**************************************************************************/
/*!
    @brief Initialises the command line using the appropriate interface
*/
/**************************************************************************/
void CLI_Init(void)
{
  // init the msg ptr
  msg_ptr = msg;

  // Show the menu
//  cliMenu();
}

/**************************************************************************/
/*!
    'help' command handler
*/
/**************************************************************************/
void cmd_help(uint8_t argc, char **argv)
{
  size_t i;

  printf("Command      Description\r\n");
  printf("-------      -----------\r\n");

  // Display full command list
  for (i=0; i < CMD_COUNT; i++)
  {
    if (!cli_tbl[i].hidden)
    {
      printf ("%-10s   %s\r\n", cli_tbl[i].command, cli_tbl[i].description);
    }
  }

  printf("Command parameters can be seen by entering: <command-name> ?\r\n");
}
